@@ -6,11 +6,12 @@
             [clj-http.headers :refer [wrap-header-map]]
             [clj-http.links :refer [wrap-links]]
             [clj-http.util :refer [opt] :as util]
+            [clojure.java.io :as io]
             [clojure.stacktrace :refer [root-cause]]
             [clojure.string :as str]
             [clojure.walk :refer [keywordize-keys prewalk]]
             [slingshot.slingshot :refer [throw+]])
-  (:import (java.io InputStream File ByteArrayOutputStream ByteArrayInputStream)
+  (:import (java.io InputStream File ByteArrayOutputStream ByteArrayInputStream EOFException BufferedReader)
            (java.net URL UnknownHostException)
            (org.apache.http.entity BufferedHttpEntity ByteArrayEntity
                                    InputStreamEntity FileEntity StringEntity)
@@ -100,11 +101,12 @@
 
 (defn ^:dynamic parse-transit
   "Resolve and apply Transit's JSON/MessagePack decoding."
-  [in type & [opts]]
+  [^InputStream in type & [opts]]
   {:pre [transit-enabled?]}
-  (let [reader (ns-resolve 'cognitect.transit 'reader)
-        read (ns-resolve 'cognitect.transit 'read)]
-    (read (reader in type (transit-read-opts opts)))))
+  (when (pos? (.available in))
+    (let [reader (ns-resolve 'cognitect.transit 'reader)
+          read (ns-resolve 'cognitect.transit 'read)]
+      (read (reader in type (transit-read-opts opts))))))
 
 (defn ^:dynamic transit-encode
   "Resolve and apply Transit's JSON/MessagePack encoding."
@@ -135,6 +137,12 @@
   {:pre [json-enabled?]}
   (apply (ns-resolve (symbol "cheshire.core") (symbol "decode-strict")) args))
 
+(defn ^:dynamic json-decode-stream
+  "Resolve and apply cheshire's json stream decoding dynamically."
+  [& args]
+  {:pre [json-enabled?]}
+  (apply (ns-resolve (symbol "cheshire.core") (symbol "decode-stream")) args))
+
 (defn ^:dynamic form-decode
   "Resolve and apply ring-codec's form decoding dynamically."
   [& args]
@@ -178,6 +186,7 @@
     {:scheme (keyword (.getProtocol url-parsed))
      :server-name (.getHost url-parsed)
      :server-port (when-pos (.getPort url-parsed))
+     :url url
      :uri (url-encode-illegal-characters (.getPath url-parsed))
      :user-info (if-let [user-info (.getUserInfo url-parsed)]
                   (util/url-decode user-info))
@@ -202,6 +211,11 @@
 (def unexceptional-status?
   #{200 201 202 203 204 205 206 207 300 301 302 303 304 307})
 
+(defn unexceptional-status-for-request?
+  [req status]
+  ((or (:unexceptional-status req) unexceptional-status?)
+   status))
+
 ;; helper methods to determine realm of a response
 (defn success?
   [{:keys [status]}]
@@ -229,7 +243,7 @@
 
 (defn- exceptions-response
   [req {:keys [status] :as resp}]
-  (if (unexceptional-status? status)
+  (if (unexceptional-status-for-request? req status)
     resp
     (if (false? (opt req :throw-exceptions))
       resp
@@ -293,7 +307,7 @@
 
 (defn- respond*
   [resp req]
-  (if (:async? req)
+  (if (opt req :async)
     ((:respond req) resp)
     resp))
 
@@ -421,76 +435,75 @@
 (defmulti coerce-response-body (fn [req _] (:as req)))
 
 (defmethod coerce-response-body :byte-array [_ resp]
-  (assoc resp :body (util/force-byte-array (:body resp))))
+  (update resp :body util/force-byte-array))
 
 (defmethod coerce-response-body :stream [_ resp]
-  (let [body (:body resp)]
-    (cond (instance? InputStream body) resp
-          ;; This shouldn't happen, but we plan for it anyway
-          (instance? (Class/forName "[B") body)
-          (assoc resp :body (ByteArrayInputStream. body)))))
+  (update resp :body util/force-stream))
+
+(defn- response-charset [response]
+  (or (-> response :content-type-params :charset)
+      "UTF-8"))
+
+(defn- can-parse-body? [{:keys [coerce] :as request} {:keys [status] :as _response}]
+  (or (= coerce :always)
+      (and (unexceptional-status-for-request? request status)
+           (or (nil? coerce)
+               (= coerce :unexceptional)))
+      (and (not (unexceptional-status-for-request? request status))
+           (= coerce :exceptional))))
+
+(defn- decode-json-body [body keyword? strict? charset]
+  (if strict?
+    ;; OPTIMIZE: When/if Cheshire gets a parse-stream-strict this won't need to go through String:
+    (json-decode-strict (util/force-string body charset) keyword?)
+    (let [^BufferedReader br (io/reader (util/force-stream body))]
+      (try
+        (.mark br 1)
+        (let [^int first-char (try (.read br) (catch EOFException _ -1))]
+          (case first-char
+            -1 nil
+            (do (.reset br)
+                (json-decode-stream br keyword?))))
+        (finally (.close br))))))
 
 (defn coerce-json-body
-  [{:keys [coerce]} {:keys [body status] :as resp} keyword? strict? & [charset]]
-  (let [^String charset (or charset (-> resp :content-type-params :charset)
-                            "UTF-8")
-        body (util/force-byte-array body)
-        decode-func (if strict? json-decode-strict json-decode)]
-    (if json-enabled?
-      (cond
-        (= coerce :always)
-        (assoc resp :body (decode-func (String. ^"[B" body charset) keyword?))
-
-        (and (unexceptional-status? status)
-             (or (nil? coerce) (= coerce :unexceptional)))
-        (assoc resp :body (decode-func (String. ^"[B" body charset) keyword?))
-
-        (and (not (unexceptional-status? status)) (= coerce :exceptional))
-        (assoc resp :body (decode-func (String. ^"[B" body charset) keyword?))
-
-        :else (assoc resp :body (String. ^"[B" body charset)))
-      (assoc resp :body (String. ^"[B" body charset)))))
+  [request {:keys [body] :as resp} keyword? strict? & [charset]]
+  (let [charset (or charset (response-charset resp))
+        body (if json-enabled?
+               (if (can-parse-body? request resp)
+                 (decode-json-body body keyword? strict? charset)
+                 (util/force-string body charset))
+               (util/force-string body charset))]
+    (assoc resp :body body)))
 
 (defn coerce-clojure-body
-  [request {:keys [body] :as resp}]
-  (let [^String charset (or (-> resp :content-type-params :charset) "UTF-8")
-        body            (util/force-byte-array body)]
+  [_request {:keys [body] :as resp}]
+  (let [charset (response-charset resp)
+        body            (util/force-string body charset)]
     (assoc resp :body (cond
                         (empty? body) nil
-                        edn-enabled? (parse-edn (String. ^"[B" body charset))
+                        edn-enabled? (parse-edn body)
                         :else (binding [*read-eval* false]
-                                (read-string (String. ^"[B" body charset)))))))
+                                (read-string body))))))
 
 (defn coerce-transit-body
-  [{:keys [transit-opts coerce] :as request} {:keys [body status] :as resp} type & [charset]]
-  (let [^String charset (or charset (-> resp :content-type-params :charset)
-                            "UTF-8")
-        body (util/force-byte-array body)]
-    (if-not (empty? body)
-      (if transit-enabled?
-        (cond
-          (= coerce :always)
-          (assoc resp :body (parse-transit (ByteArrayInputStream. body) type transit-opts))
-
-          (and (unexceptional-status? status)
-               (or (nil? coerce) (= coerce :unexceptional)))
-          (assoc resp :body (parse-transit (ByteArrayInputStream. body) type transit-opts))
-
-          (and (not (unexceptional-status? status)) (= coerce :exceptional))
-          (assoc resp :body (parse-transit (ByteArrayInputStream. body) type transit-opts))
-
-          :else (assoc resp :body (String. ^"[B" body charset)))
-        (assoc resp :body (String. ^"[B" body charset)))
-      (assoc resp :body nil))))
+  [{:keys [transit-opts] :as request}
+   {:keys [body] :as resp} type & [charset]]
+  (let [charset (or charset (response-charset resp))
+        body (if transit-enabled?
+               (if (can-parse-body? request resp)
+                 (parse-transit (util/force-stream body) type transit-opts)
+                 (util/force-string body charset))
+               nil)]
+    (assoc resp :body body)))
 
 (defn coerce-form-urlencoded-body
-  [request {:keys [body] :as resp}]
-  (let [^String charset (or (-> resp :content-type-params :charset) "UTF-8")
-        body-bytes (util/force-byte-array body)]
-    (if ring-codec-enabled?
-      (assoc resp :body (-> (String. ^"[B" body-bytes charset)
-                            form-decode keywordize-keys))
-      (assoc resp :body (String. ^"[B" body-bytes charset)))))
+  [_request {:keys [body] :as resp}]
+  (let [charset (response-charset resp)
+        body (util/force-string body charset)]
+    (assoc resp :body (if ring-codec-enabled?
+                        (-> body form-decode keywordize-keys)
+                        body))))
 
 (defmulti coerce-content-type (fn [req resp] (:content-type resp)))
 
@@ -548,10 +561,7 @@
 
 (defmethod coerce-response-body :default
   [{:keys [as]} {:keys [body] :as resp}]
-  (let [body-bytes (util/force-byte-array body)]
-    (cond
-      (string? as)  (assoc resp :body (String. ^"[B" body-bytes ^String as))
-      :else (assoc resp :body (String. ^"[B" body-bytes "UTF-8")))))
+  (assoc resp :body (util/force-string body (if (string? as) as "UTF-8"))))
 
 (defn- output-coercion-response
   [req {:keys [body] :as resp}]
@@ -966,13 +976,12 @@
     request))
 
 (defn- nest-params-request
-  [{:keys [content-type] :as req}]
-  (if (or (nil? content-type)
-          (= content-type :x-www-form-urlencoded))
+  [{:keys [flatten-nested-keys] :as req}]
+  (if (seq flatten-nested-keys)
     (reduce
      nest-params
      req
-     [:query-params :form-params])
+     flatten-nested-keys)
     req))
 
 (defn wrap-nested-params
@@ -984,6 +993,33 @@
     ([req respond raise]
      (client (nest-params-request req) respond raise))))
 
+(defn- nested-keys-to-flatten
+  [{:keys [flatten-nested-keys] :as req}]
+  (when (and (or (not (nil? (opt req :ignore-nested-query-string)))
+                 (not (nil? (opt req :flatten-nested-form-params))))
+             flatten-nested-keys)
+    (throw (IllegalArgumentException.
+            (str "only :flatten-nested-keys or :ignore-nested-query-string/"
+                 ":flatten-nested-keys may be specified, not both"))))
+  (let [iqs-key (when-not (opt req :ignore-nested-query-string) :query-params)
+        ifp-key (when (opt req :flatten-nested-form-params) :form-params)]
+    (or flatten-nested-keys
+        (remove nil? (list iqs-key ifp-key)))))
+
+(defn wrap-flatten-nested-params
+  "Middleware wrapping options for whether or not to flatten `:query-params` and
+  `:form-params`. Modifies the request by adding a `:flatten-nested-keys`
+  sequence of the nested keys that will be flattened."
+  [client]
+  (fn
+    ([req]
+     (client
+      (assoc req :flatten-nested-keys (nested-keys-to-flatten req))))
+    ([req respond raise]
+     (client
+      (assoc req :flatten-nested-keys (nested-keys-to-flatten req))
+      respond raise))))
+
 (defn- url-request
   [req]
   (if-let [url (:url req)]
@@ -1051,54 +1087,9 @@
                #(respond (request-timing-response % start))
                raise)))))
 
-(defn- async-pooling-request
-  [req release]
-  (let [handler (:oncancel req)]
-    (assoc req :oncancel
-           (fn []
-             (release)
-             (when handler
-               (handler))))))
-
-(def ^:dynamic *pooling-info*
-  "The pooling-info used in pooling function"
-  nil)
-
-(defn wrap-async-pooling
-  "Middleware that handle pooling async request. It use the value of key
-  :pooling-info, the value is a map contains three key-value
-
-  :conn-mgr - connection manager used by the HttpAsyncClient
-  :allocate - a function with no args, will be invoked when connection allocate
-  :release  - a function with no args, will be invoked when connection finished
-
-  You must shutdown the conn-mgr when you don't need it."
-  [client]
-  (fn
-    ([req]
-     (client req))
-    ([req respond raise]
-     (if-let [pooling-info (or *pooling-info* (:pooling-info req))]
-       (let [{:keys [allocate release conn-mgr]} pooling-info]
-         (binding [conn/*async-connection-manager* conn-mgr]
-           (allocate)
-           (try
-             (client (async-pooling-request req release)
-                     (fn [resp]
-                       (respond (assoc resp :pooling-info pooling-info))
-                       (release))
-                     (fn [ex]
-                       (release)
-                       (raise ex)))
-             (catch Throwable ex
-               (release)
-               (throw ex)))))
-       (client req respond raise)))))
-
 (def default-middleware
   "The default list of middleware clj-http uses for wrapping requests."
   [wrap-request-timing
-   wrap-async-pooling
    wrap-header-map
    wrap-query-params
    wrap-basic-auth
@@ -1117,6 +1108,7 @@
    wrap-content-type
    wrap-form-params
    wrap-nested-params
+   wrap-flatten-nested-params
    wrap-method
    wrap-cookies
    wrap-links
@@ -1172,8 +1164,8 @@
      (throw (IllegalArgumentException. "Host URL cannot be nil"))))
 
 (defn- request*
-  [{:keys [async?] :as req} [respond raise]]
-  (if async?
+  [req [respond raise]]
+  (if (opt req :async)
     (if (some nil? [respond raise])
       (throw (IllegalArgumentException.
               "If :async? is true, you must pass respond and raise"))
@@ -1310,32 +1302,46 @@
     options))
 
 (defmacro with-async-connection-pool
+  "Macro to execute the body using a connection manager. Creates a
+  PoolingNHttpClientConnectionManager to use for all requests within the body of
+  the expression. An option map is allowed to set options for the connection
+  manager.
+
+  Handles the same options as `with-connection-pool` plus:
+  :io-config which should be a map containing some of the following keys:
+
+  :connect-timeout - int the default connect timeout value for connection
+    requests (default 0, meaning no timeout)
+  :interest-op-queued - boolean, whether or not I/O interest operations are to
+    be queued and executed asynchronously or to be applied to the underlying
+    SelectionKey immediately (default false)
+  :io-thread-count - int, the number of I/O dispatch threads to be used
+    (default is the number of available processors)
+  :rcv-buf-size - int the default value of the SO_RCVBUF parameter for
+    newly created sockets (default is 0, meaning the system default)
+  :select-interval - long, time interval in milliseconds at which to check for
+    timed out sessions and session requests (default 1000)
+  :shutdown-grace-period - long, grace period in milliseconds to wait for
+    individual worker threads to terminate cleanly (default 500)
+  :snd-buf-size - int, the default value of the SO_SNDBUF parameter for
+    newly created sockets (default is 0, meaning the system default)
+  :so-keep-alive - boolean, the default value of the SO_KEEPALIVE parameter for
+    newly created sockets (default false)
+  :so-linger - int, the default value of the SO_LINGER parameter for
+    newly created sockets (default -1)
+  :so-timeout - int, the default socket timeout value for I/O operations
+    (default 0, meaning no timeout)
+  :tcp-no-delay - boolean, the default value of the TCP_NODELAY parameter for
+    newly created sockets (default true)
+
+  If the value 'nil' is specified or the value is not set, the default value
+  will be used."
   [opts & body]
-  `(let [cm# (conn/make-reuseable-async-conn-manager ~opts)
-         count# (atom 0)
-         all-requested# (atom false)
-         p-info# {:conn-mgr cm#
-                  :allocate (fn [] (swap! count# inc))
-                  :release  (fn [] (swap! count# dec))}
-         ;; A http client hold the conn-mgr and start the io-reactor.
-         ;; In this context any other client's ConnectionManagerShared will be
-         ;; set to true.
-         holder-client# (-> (HttpAsyncClients/custom)
-                            (.setConnectionManager cm#)
-                            (.build)
-                            (.start))]
-     (add-watch count# :close-conn-mgr
-                (fn [key# identity# old# new#]
-                  (if (and (not= old# new#) (<= new# 0) @all-requested#)
-                    (.shutdown
-                     ^PoolingNHttpClientConnectionManager
-                     cm#))))
-     (binding [*pooling-info* p-info#]
+  `(let [cm# (conn/make-reuseable-async-conn-manager ~opts)]
+     (binding [conn/*async-connection-manager* cm#]
        (try
          ~@body
          (finally
-           (swap! all-requested# not)
-           (if (= 0 @count#)
-             (.shutdown
-              ^PoolingNHttpClientConnectionManager
-              cm#)))))))
+           (.shutdown
+            ^PoolingNHttpClientConnectionManager
+            cm#))))))
