@@ -12,21 +12,10 @@
              BasicAWSCredentials
              BasicSessionCredentials
              DefaultAWSCredentialsProviderChain]
-           [com.amazonaws.auth.profile
-             ProfileCredentialsProvider]
+           com.amazonaws.auth.profile.ProfileCredentialsProvider
            [com.amazonaws.regions
              Region
              Regions]
-           com.amazonaws.services.cloudsearchdomain.AmazonCloudSearchDomainClient
-           com.amazonaws.services.lambda.AWSLambdaClient
-           [com.amazonaws.services.s3
-             AmazonS3Client
-             AmazonS3EncryptionClient]
-           [com.amazonaws.services.s3.model
-             CryptoConfiguration
-             EncryptionMaterials
-             StaticEncryptionMaterialsProvider]
-           com.amazonaws.services.s3.transfer.TransferManager
            org.joda.time.DateTime
            org.joda.time.base.AbstractInstant
            java.io.File
@@ -37,20 +26,31 @@
            java.lang.reflect.Modifier
            java.math.BigDecimal
            java.math.BigInteger
+           java.nio.ByteBuffer
            java.text.ParsePosition
            java.text.SimpleDateFormat
            java.util.Date))
 
 (defonce ^:private credential (atom {}))
 
+(defonce ^:private client-config (atom {}))
+
 (def ^:dynamic ^:private *credentials* nil)
 
+(def ^:dynamic ^:private *client-config* nil)
+
 (def ^:dynamic ^:private *client-class* nil)
 
 (def ^:private date-format (atom "yyyy-MM-dd"))
 
 (def ^:private root-unwrapping (atom false))
 
+(defn- invoke-constructor
+  [class-name arg-vec]
+  (Reflector/invokeConstructor
+    (Class/forName class-name)
+    (into-array Object arg-vec)))
+
 (defn set-root-unwrapping!
   "Enables JSON-like root unwrapping of singly keyed
   top level maps.
@@ -96,16 +96,6 @@
    :message      (.getMessage e)
    :stack-trace  (stack->string e)})
 
-; Java methods on the AWS*Client class which won't be exposed
-(def ^:private excluded
-  #{:invoke
-    :init
-    :set-endpoint
-    :get-cached-response-metadata
-    :get-service-abbreviation})
-    ; addRequestHandler???
-
-
 (defn keys->cred
   [access-key secret-key & [endpoint]]
   (let [credential {:access-key access-key
@@ -122,6 +112,12 @@
   ([access-key secret-key & [endpoint]]
    (defcredential (keys->cred access-key secret-key endpoint))))
 
+(defn defclientconfig
+  "Specify the default Client configuration to use on
+  subsequent requests."
+  [config]
+  (reset! client-config config))
+
 (defmacro with-credential
   "Per invocation binding of credentials for ad-hoc
   service calls using alternate user/password combos
@@ -134,22 +130,30 @@
                  cred#))]
     (do ~@body)))
 
+(defmacro with-client-config
+  "Per invocation binding of client-config for ad-hoc
+  service calls using alternate client configuration."
+  [config & body]
+  `(binding [*client-config* config]
+     (do ~@body)))
+
 (declare new-instance)
 
 (defn- create-client
-  [aws-client credentials configuration]
+  [clazz credentials configuration]
   (if (every? nil? [credentials configuration])
-    (new-instance aws-client)
-    ; TransferManager is the only client to date that doesn't
-    ; accept AWSCredentialsProviders
-    (if (= aws-client TransferManager)
-        (TransferManager. (create-client AmazonS3Client
-                                         credentials
-                                         configuration))
-        (Reflector/invokeConstructor aws-client
-                                     (into-array Object
-                                                 (filter (comp not nil?)
-                                                         [credentials configuration]))))))
+    (new-instance (Class/forName clazz))
+    ; TransferManager is the only client to date that doesn't accept AWSCredentialsProviders
+    (if (= (.getSimpleName clazz) "TransferManager")
+        (invoke-constructor
+          "com.amazonaws.services.s3.transfer.TransferManager"
+          [(create-client (Class/forName "com.amazonaws.services.s3.AmazonS3Client")
+                          credentials
+                          configuration)])
+        (invoke-constructor (.getName clazz)
+                            (->> [credentials configuration]
+                                 (filter (comp not nil?))
+                                 vec)))))
 
 (defn get-credentials
   [credentials]
@@ -195,58 +199,84 @@
   (when (associative? configuration)
     (create-bean ClientConfiguration configuration)))
 
-(defn- encryption-client*
+(defn- set-endpoint!
+  [client credentials]
+  (when-let [endpoint (or (:endpoint credentials)
+                          (System/getenv "AWS_DEFAULT_REGION"))]
+    (if (contains? (fmap #(-> % str/upper-case (str/replace "_" ""))
+                         (apply hash-set (seq (Regions/values))))
+                   (-> (str/upper-case endpoint)
+                       (str/replace "-" "")))
+        (try
+          (->> (-> (str/upper-case endpoint)
+                   (str/replace "-" "_"))
+               Regions/valueOf
+               Region/getRegion
+               (.setRegion client))
+          (catch NoSuchMethodException e
+            (println e)))
+        (.setEndpoint client endpoint))))
+
+(defn encryption-client*
   [encryption credentials configuration]
   (let [creds     (get-credentials credentials)
         config    (get-client-configuration configuration)
         key       (or (:secret-key encryption)
                       (:key-pair encryption))
-        crypto    (CryptoConfiguration.)
-        em        (EncryptionMaterials. key)
-        materials (StaticEncryptionMaterialsProvider. em)]
-    (if-let [provider (:provider encryption)]
-        (.withCryptoProvider crypto provider))
-    (if config
-      (AmazonS3EncryptionClient. creds materials config crypto)
-      (AmazonS3EncryptionClient. creds materials crypto))))
+        crypto    (invoke-constructor
+                    "com.amazonaws.services.s3.model.CryptoConfiguration" [])
+        em        (invoke-constructor
+                    "com.amazonaws.services.s3.model.EncryptionMaterials"
+                    [key])
+        materials (invoke-constructor
+                    "com.amazonaws.services.s3.model.StaticEncryptionMaterialsProvider"
+                    [em])
+        _         (if-let [provider (:provider encryption)]
+                    (.withCryptoProvider crypto provider))
+        client    (if config
+                      (invoke-constructor
+                        "com.amazonaws.services.s3.AmazonS3EncryptionClient"
+                        [creds materials config crypto])
+                      (invoke-constructor
+                        "com.amazonaws.services.s3.AmazonS3EncryptionClient"
+                        [creds materials crypto]))]
+    (set-endpoint! client credentials)
+    client))
 
-(defn- amazon-client*
+(swap! client-config assoc :encryption-client-fn (memoize encryption-client*))
+
+(defn amazon-client*
   [clazz credentials configuration]
   (let [aws-creds  (get-credentials credentials)
         aws-config (get-client-configuration configuration)
         client     (create-client clazz aws-creds aws-config)]
-    (when-let [endpoint (or (:endpoint credentials)
-                            (System/getenv "AWS_DEFAULT_REGION"))]
-      (if (contains? (fmap #(-> % str/upper-case (str/replace "_" ""))
-                           (apply hash-set (seq (Regions/values))))
-                     (-> (str/upper-case endpoint)
-                         (str/replace "-" "")))
-          (try
-            (->> (-> (str/upper-case endpoint)
-                     (str/replace "-" "_"))
-                 Regions/valueOf
-                 Region/getRegion
-                 (.setRegion client))
-            (catch NoSuchMethodException e
-              (println e)))
-          (.setEndpoint client endpoint)))
+    (set-endpoint! client credentials)
     client))
 
-(def ^:private encryption-client
-  (memoize encryption-client*))
+(swap! client-config assoc :amazon-client-fn (memoize amazon-client*))
 
-(def ^:private amazon-client
-  (memoize amazon-client*))
+(defn- keyword-converter
+  "Given something that tokenizes a string into parts, turn it into
+  a :kebab-case-keyword."
+  [separator-regex]
+  (fn [s]
+    (->> (str/split s separator-regex)
+         (map str/lower-case)
+         (interpose \-)
+         str/join
+         keyword)))
 
-
-(defn- camel->keyword
+(def ^:private camel->keyword
   "from Emerick, Grande, Carper 2012 p.70"
-  [s]
-  (->> (str/split s #"(?<=[a-z])(?=[A-Z])")
-       (map str/lower-case)
-       (interpose \-)
-       str/join
-       keyword))
+  (keyword-converter #"(?<=[a-z])(?=[A-Z])"))
+
+(def ^:private camel->keyword2
+  "Like [[camel->keyword]], but smarter about acronyms and concepts like iSCSI
+  and OpenID which should be treated as a single concept."
+  (comp
+   (keyword-converter #"(?<!(^|[A-Z]))(?=[A-Z])|(?<!^)(?=[A-Z][a-z])")
+   #(str/replace % #"(?i)iscsi" "ISCSI")
+   #(str/replace % #"(?i)openid" "OPENID")))
 
 (defn- keyword->camel
   [kw]
@@ -256,13 +286,11 @@
          (fmap str/capitalize)
          str/join)))
 
-
 (defn- aws-package?
   [clazz]
   (->> (.getName clazz)
        (re-find #"com\.amazonaws\.services")
-       nil?
-       not))
+       some?))
 
 (defn to-date
   [date]
@@ -294,22 +322,32 @@
         (.invoke type (make-array Object 0)))))
 
 ; assoc java Class to Clojure cast functions
-(defonce ^:private coercions (atom
-  {String     str
-   Integer    int
-   Long       long
-   Boolean    boolean
-   Double     double
-   Float      float
-   BigDecimal bigdec
-   BigInteger bigint
-   Date       to-date
-   File       to-file
-   "int"      int
-   "long"     long
-   "double"   double
-   "float"    float
-   "boolean"  boolean}))
+(defonce ^:private coercions
+  (->> [:String :Integer :Long :Double :Float]
+       (reduce
+         (fn [m e]
+           (let [arr (str "[Ljava.lang." (name e) ";")
+                 clazz (Class/forName arr)]
+             (assoc m clazz into-array))) {})
+       (merge {
+         String     str
+         Integer    int
+         Long       long
+         Boolean    boolean
+         Double     double
+         Float      float
+         BigDecimal bigdec
+         BigInteger bigint
+         Date       to-date
+         File       to-file
+         Region     #(Region/getRegion (Regions/fromName %))
+         ByteBuffer #(-> % str .getBytes ByteBuffer/wrap)
+         "int"      int
+         "long"     long
+         "double"   double
+         "float"    float
+         "boolean"  boolean})
+       atom))
 
 (defn register-coercions
   "Accepts key/value pairs of class/function, which defines
@@ -320,7 +358,7 @@
 
 (defn coerce-value
   "Coerces the supplied stringvalue to the required type as
-  defined by the AWS method signature. String or keyword 
+  defined by the AWS method signature. String or keyword
   conversion to Enum types (e.g. via valueOf()) is supported."
   [value type]
   (let [value (if (keyword? value) (name value) value)]
@@ -359,6 +397,7 @@
 (defn best-constructor
   "Prefer no-arg ctor if one exists, else the first found."
   [clazz]
+  {:pre [clazz]}
   (let [ctors (.getConstructors clazz)]
     (or
       (some
@@ -376,6 +415,7 @@
   the check for contructor args here, as the rest of the
   AWS api contains strictly no-arg ctor JavaBeans."
   [clazz]
+  {:pre [clazz]}
   (let [ctor (best-constructor clazz)
         arr  (constructor-args ctor)]
     (.newInstance ctor arr)))
@@ -383,18 +423,22 @@
 (defn- unwind-types
   [depth param]
   (if (instance? ParameterizedType param)
-      (let [f (partial unwind-types (inc depth))]
-        (-> param
-            (.getActualTypeArguments)
-            last
-            f))
+      (let [f (partial unwind-types (inc depth))
+            types (-> param .getActualTypeArguments)
+            t (-> types last f)]
+        (if (and (instance? ParameterizedType (last types))
+                 (.contains (-> types last str) "java.util")
+                 (.contains (-> types last str) "java.lang"))
+          {:type [(-> types last .getRawType)]
+           :depth depth}
+          t))
       {:type [param]
        :depth depth}))
 
 (defn- paramter-types
   [method]
   (let [types (seq (.getGenericParameterTypes method))
-        param (first types)
+        param (last types)
         rval  {:generic types}]
     (if (instance? ParameterizedType param)
         (let [t (unwind-types 1 param)]
@@ -423,20 +467,29 @@
    in the DynamoDBV2Client.
    http://docs.aws.amazon.com/AWSJavaSDK/latest/javadoc/com/amazonaws/services/dynamodbv2/model/DeleteItemRequest.html#setKey(java.util.Map.Entry, java.util.Map.Entry)
    http://docs.aws.amazon.com/AWSJavaSDK/latest/javadoc/com/amazonaws/services/dynamodbv2/model/GetItemRequest.html#setKey(java.util.Map.Entry, java.util.Map.Entry)"
-  [method name getter?]
+  [method name value]
   (let [args (.getParameterTypes method)]
     (and (= name (normalized-name (.getName method)))
-         (case getter?
-           true  (= 0 (count args))
-           false (and (< 0 (count args))
-                      (not (and (= 2 (count args))
-                                (every? (partial = java.util.Map$Entry)
-                                        args))))))))
+         (if (empty? value)
+             (= 0 (count args))
+             (and (< 0 (count args))
+                  (or (= (count args) (count (flatten value)))
+                      (and (coll? value)
+                           (= 1 (count args))
+                           (or (contains? @coercions (first args))
+                               (.isArray (first args))
+                               (and (.getPackage (first args))
+                                    (.startsWith
+                                      (.getName (.getPackage (first args)))
+                                      "java.util")))))
+                  (not (and (= 2 (count args))
+                            (every? (partial = java.util.Map$Entry)
+                                    args))))))))
 
 (defn- accessor-methods
-  [class-methods name getter?]
+  [class-methods name value]
   (reduce
-    #(if (matches? %2 name getter?)
+    #(if (matches? %2 name value)
       (conj %1 %2)
       %1)
     []
@@ -448,7 +501,7 @@
       (.getMethods)
       (accessor-methods
         (.toLowerCase (keyword->camel k))
-        (empty? v))))
+        v)))
 
 (defn to-java-coll
   "Need this only because S3 methods actually try to
@@ -470,7 +523,8 @@
     (into-array
       ;; misbehaving S3Client mutates the coll
       (if (and (coll? v)
-            (= AmazonS3Client *client-class*))
+               (not (nil? *client-class*))
+               (= "AmazonS3Client" (.getSimpleName *client-class*)))
         (if (and (> (count (.getParameterTypes method)) 1)
                  (sequential? v))
           (to-java-coll v)
@@ -486,6 +540,7 @@
 (defn- populate
   [types key props]
   (let [type (-> types key last)]
+    (assert type (str "Bad data type - there is no " key " in " types))
     (if (contains? @coercions type)
       (coerce-value props type)
       (set-fields (new-instance type) props))))
@@ -496,28 +551,36 @@
   (let [type (last (or (:actual types)
                        (:generic types)))
         pp   (partial populate types :actual)]
-    (if (aws-package? type)
-        (if (map? col)
-            (if (contains? types :actual)
-                (if (< (:depth types) 3)
-                    (apply assoc {}
-                           (interleave (fmap kw->str (apply vector (keys col)))
-                                       (fmap pp (apply vector (vals col)))))
-                    (apply assoc {}
-                           (interleave (fmap kw->str (apply vector (keys col)))
-                                       [(fmap #(populate {:generic [type]}
-                                                              :generic
-                                                              %)
-                                                  (first (apply vector (vals col))))])))
-                (populate types :generic col))
-            (if (and (contains? types :actual)
-                     (= (:depth types) 3))
-                (fmap #(fmap pp %) col)
-                (fmap pp col)))
-        (if (and (contains? types :actual)
-                 (aws-package? type))
-            (fmap pp col)
-            (fmap #(coerce-value % type) col)))))
+    (try
+      (if (aws-package? type)
+       (if (map? col)
+         (if (contains? types :actual)
+           (if (< (:depth types) 3)
+             (apply assoc {}
+                    (interleave (fmap kw->str (apply vector (keys col)))
+                                (fmap pp (apply vector (vals col)))))
+             (apply assoc {}
+                    (interleave (fmap kw->str (apply vector (keys col)))
+                                [(fmap #(populate {:generic [type]}
+                                                  :generic
+                                                  %)
+                                       (first (apply vector (vals col))))])))
+           (populate types :generic col))
+         (if (and (contains? types :actual)
+                  (= (:depth types) 3))
+           (fmap #(fmap pp %) col)
+           (fmap pp col)))
+       (if (and (contains? types :actual)
+                (aws-package? type))
+         (fmap pp col)
+         (fmap #(coerce-value % type) col)))
+      (catch Throwable e
+        (throw (RuntimeException. (str
+                                    "Failed to create an instance of "
+                                    (.getName type)
+                                    " from " col
+                                    " due to " e
+                                    ". Make sure the data matches an existing constructor and setters.")))))))
 
 (defn- invoke-method
   [pojo v method]
@@ -539,8 +602,14 @@
    to the corresponding method calls on the object graph."
   [pojo args]
   (doseq [[k v] args]
-    (->> (find-methods pojo k v)
-         (some (partial invoke-method pojo v))))
+    (try
+      (->> (find-methods pojo k v)
+           (some (partial invoke-method pojo v)))
+      (catch Throwable e
+        (throw (ex-info
+                 (str "Error setting " k ": " (.getMessage e) ". Perhaps the value isn't compatible with the setter?")
+                 {:property k, :value v}
+                 e)))))
   pojo)
 
 (defn- create-bean
@@ -553,9 +622,10 @@
    Amazon*Client class. (Note that we assume all AWS
    service calls take at most a single argument.)"
   [method args]
-  (-> (.getParameterTypes method)
-      first
-      (create-bean args)))
+  (let [clazz (first (.getParameterTypes method))]
+    (if (contains? @coercions clazz)
+        (coerce-value (into {} args) clazz)
+        (create-bean clazz args))))
 
 
 (defprotocol IMarshall
@@ -572,7 +642,9 @@
   [method]
   (let [name (.getName method)
         type (.getName (.getReturnType method))]
-    (or (.startsWith name "get")
+    (or (and
+          (.startsWith name "get")
+          (= 0 (count (.getParameterTypes method))))
         (and
           (.startsWith name "is")
           (= "boolean" type)))))
@@ -613,7 +685,7 @@
 (extend-protocol IMarshall
   nil
   (marshall [obj] nil)
-  
+
   java.util.Map
   (marshall [obj]
     (if-not (empty? obj)
@@ -623,21 +695,21 @@
                 (apply vector (keys obj)))
           (fmap marshall
                 (apply vector (vals obj)))))))
-  
+
   java.util.Collection
   (marshall [obj]
     (if (instance? clojure.lang.IPersistentSet obj)
       obj
       (fmap marshall (apply vector obj))))
-  
+
   java.util.Date
   (marshall [obj] (DateTime. (.getTime obj)))
-  
+
   ; `false` boolean objects (i.e. (Boolean. false)) come out of e.g.
   ; .doesBucketExist, which wreak havoc on Clojure truthiness
   Boolean
   (marshall [obj] (when-not (nil? obj) (.booleanValue obj)))
-  
+
   Object
   (marshall [obj]
     (if (aws-package? (class obj))
@@ -706,6 +778,7 @@
                    (map? (first args)))
                (or (contains? (first args) :access-key)
                    (contains? (first args) :endpoint)
+                   (contains? (first args) :profile)
                    (contains? (first args) :client-config)))
           (instance? AWSCredentialsProvider (first args))
           (instance? AWSCredentials (first args)))
@@ -721,33 +794,41 @@
                (if (seq m) m {}))}
       :default {:args args})))
 
-(declare candidate-client)
-
-(defn- transfer-manager*
+(defn transfer-manager*
   [credential client-config crypto]
-  (TransferManager. (if crypto
-                      (encryption-client crypto credential client-config)
-                      (amazon-client AmazonS3Client credential client-config))))
+  (let [encryption-client (:encryption-client-fn client-config)
+        amazon-client (:amazon-client-fn client-config)]
+    (invoke-constructor
+      "com.amazonaws.services.s3.transfer.TransferManager"
+      (if crypto
+        [(encryption-client crypto credential client-config)]
+        [(amazon-client (Class/forName "com.amazonaws.services.s3.AmazonS3Client")
+                        credential
+                        client-config)]))))
 
-(def ^:private transfer-manager
-  (memoize transfer-manager*))
+(swap! client-config assoc :transfer-manager-fn (memoize transfer-manager*))
 
-(defn- candidate-client
+(defn candidate-client
   [clazz args]
-  (let [credential (if (map? (:credential args))
-                             (merge @credential (:credential args))
-                             (or (:credential args) @credential))
-        client-config (:client-config args)
+  (let [cred-bound (or *credentials* (:credential args))
+        credential (if (map? cred-bound)
+                             (merge @credential cred-bound)
+                             (or cred-bound @credential))
+        config-bound (or *client-config* (:client-config args))
+        client-config (merge @client-config config-bound)
+        encryption-client (:encryption-client-fn client-config)
+        amazon-client (:amazon-client-fn client-config)
+        transfer-manager (:transfer-manager-fn client-config)
         crypto (if (even? (count (:args args)))
                    (:encryption (apply hash-map (:args args))))
-        client  (if (and crypto (or (= clazz AmazonS3Client)
-                                    (= clazz TransferManager)))
+        client  (if (and crypto (or (= (.getSimpleName clazz) "AmazonS3Client")
+                                    (= (.getSimpleName clazz) "TransferManager")))
                     (delay (encryption-client crypto credential client-config))
                     (delay (amazon-client clazz credential client-config)))]
-        (if (= clazz TransferManager)
+        (if (= (.getSimpleName clazz) "TransferManager")
             (transfer-manager credential client-config crypto)
             @client)))
-        
+
 
 (defn- fn-call
   "Returns a function that reflectively invokes method on
@@ -760,10 +841,7 @@
           client  (delay (candidate-client clazz args))]
       (fn []
         (try
-          (let [c (if (thread-bound? #'*credentials*)
-                      (candidate-client clazz (assoc args :credential *credentials*))
-                      @client)
-                java (.invoke method c arg-arr)
+          (let [java (.invoke method @client arg-arr)
                 cloj (marshall java)]
             (if (and
                   @root-unwrapping
@@ -819,8 +897,7 @@
   "Finds the appropriate method to invoke in cases where
   the Amazon*Client has overloaded methods by arity or type."
   [methods & arg]
-  (let [args (:args (args-from arg))
-        methods (filter #(not (Modifier/isPrivate (.getModifiers %))) methods)]
+  (let [args (:args (args-from arg))]
     (or (some (partial types-match-args args) methods)
         (choose-from (possible-methods methods args)))))
 
@@ -829,7 +906,9 @@
    derived from the java.lang.reflect.Method(s). Overloaded
    methods will yield a variadic Clojure function."
   [client ns fname methods]
-  (intern ns (symbol (name fname))
+  (intern ns (with-meta (symbol (name fname))
+               {:amazonica/client client
+                :amazonica/methods methods})
     (fn [& args]
       (if-let [method (best-method methods args)]
         (if-not args
@@ -840,27 +919,35 @@
                          (name fname) args)))))))
 
 (defn- client-methods
-  "Returns a map with keys of idiomatic Clojure hyphenated
-   keywords corresponding to the public Java method names of
-   the class argument, vals are vectors of
-   java.lang.reflect.Methods."
+  "Returns a map with keys of idiomatic Clojure hyphenated keywords
+  corresponding to the public Java method names of the class argument, vals are
+  vectors of java.lang.reflect.Methods."
   [client]
-  (reduce
-    (fn [col method]
-      (let [fname (camel->keyword (.getName method))]
-        (if (and (contains? excluded fname)
-                 (not= client AWSLambdaClient)
-                 (not= client AmazonCloudSearchDomainClient))
-            col
-            (if (contains? col fname)
-                (update-in col [fname] conj method)
-                (assoc col fname [method])))))
-    {}
-    (.getDeclaredMethods client)))
+  (->> (.getDeclaredMethods client)
+       (remove (fn [method]
+                 (let [mods (.getModifiers method)]
+                   (or (.isSynthetic method)
+                       (Modifier/isPrivate mods)
+                       (Modifier/isProtected mods)
+                       (Modifier/isStatic mods)))))
+       (group-by #(camel->keyword (.getName %)))))
+
+(defn- show-functions [ns]
+  (intern ns (symbol "show-functions")
+    (fn []
+      (->> (ns-publics ns)
+           sort
+           (map (comp println first))))))
 
 (defn set-client
   "Intern into the specified namespace all public methods
    from the Amazon*Client class as Clojure functions."
   [client ns]
-  (doseq [[k v] (client-methods client)]
-    (intern-function client ns k v)))
+  (show-functions ns)
+  (intern ns 'client-class client)
+  (doseq [[fname methods] (client-methods client)
+          :let [the-var (intern-function client ns fname methods)
+                fname2 (-> methods first .getName camel->keyword2)]]
+    (when (not= fname fname2)
+      (let [the-var2 (intern-function client ns fname2 methods)]
+        (alter-meta! the-var assoc :amazonica/deprecated-in-favor-of the-var2)))))
