@@ -1,4 +1,4 @@
-(defproject riemann "0.3.6"
+(defproject riemann "0.3.8"
   :description
 "A network event stream processor. Intended for analytics, metrics, and alerting; and to glue various monitoring systems together."
   :url "https://github.com/riemann/riemann"
@@ -11,17 +11,18 @@
     [org.clojure/algo.generic "0.1.2"]
     [org.clojure/clojure "1.9.0"]
     [org.clojure/math.numeric-tower "0.0.4"]
-    [org.clojure/tools.logging "0.4.0"]
+    [org.clojure/tools.logging "1.2.1"]
     [org.clojure/tools.nrepl "0.2.13"]
     [org.clojure/core.cache "0.6.5"]
     [org.clojure/data.priority-map "0.0.10"]
     [org.clojure/java.classpath "0.3.0"]
-    [org.slf4j/log4j-over-slf4j "1.7.25"]
+    [org.slf4j/log4j-over-slf4j "1.7.32"]
     [javax.xml.bind/jaxb-api "2.4.0-b180830.0359"]
     [ch.qos.logback/logback-classic "1.2.3"]
     [com.github.juise/logstash-logback-layout "1.0"]
-    [net.logstash.logback/logstash-logback-encoder "4.11"]
-    [org.influxdb/influxdb-java "2.8"]
+    [net.logstash.logback/logstash-logback-encoder "6.4"]
+    [org.influxdb/influxdb-java "2.21"]
+    [com.influxdb/influxdb-client-java "2.2.0"]
     [com.cemerick/pomegranate "1.0.0"
      :exclusions [org.codehaus.plexus/plexus-utils]]
     ; for pomegranate
@@ -45,7 +46,9 @@
     [amazonica "0.3.95" :exclusions [joda-time]]
     [spootnik/kinsky "0.1.20"]
     [pjstadig/humane-test-output "0.8.3"]
-    [com.novemberain/langohr "5.1.0"]]
+    [com.novemberain/langohr "5.1.0"]
+    [com.fasterxml.jackson.core/jackson-core "2.10.0"]
+    [com.fasterxml.jackson.core/jackson-databind "2.10.0"]]
   :plugins [[lein-codox "0.10.6"]
             [lein-difftest "2.0.0"]
             [lein-ancient "0.6.15"]
@@ -77,6 +80,7 @@
                    :sns :sns
                    :graphite :graphite
                    :influxdb :influxdb
+                   :influxdb2 :influxdb2
                    :kairosdb :kairosdb
                    :librato :librato
                    :hipchat :hipchat
